/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.Objects;

/**
 *
 * @author johan
 */
public abstract class AbstractEntity implements Entity, Serializable {

    private AgroURI uri;
    private transient PropertyChangeSupport propertyChangeSupport;

    // FIXME: remove once all implementing entities can use the URI constructor
    @Deprecated
    public AbstractEntity() {
    }

    public AbstractEntity(AgroURI uri) {
        this.uri = uri;
    }

    public AbstractEntity(EnterpriseIdProvider eip, ItemIdType itemIdType) {
        this(new AgroURIFactory(eip).createURI(itemIdType));
    }

    @Override
    public String getItemIdNumber() {
        return uri.getItemIdNumber();
    }

    @Override
    public ItemIdType getItemIdType() {
        return uri.getItemIdType();
    }

    @Override
    public AgroURI getURI() {
        return uri;
    }

    /**
     * 
     * @param uri
     * @deprecated URI should be immutable.
     */
    @Deprecated
    protected void setURI(AgroURI uri) {
        this.uri = uri;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 13 * hash + Objects.hashCode(this.uri);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final AbstractEntity other = (AbstractEntity) obj;

        return Objects.equals(this.getURI(), other.getURI());
    }

    protected PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    @Override
    public void addPropertyChangeListener(String property, PropertyChangeListener listener) {
        assert listener != null;
        getPropertyChangeSupport().addPropertyChangeListener(property, listener);
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    @Override
    public void removePropertyChangeListener(String property, PropertyChangeListener listener) {
        getPropertyChangeSupport().removePropertyChangeListener(property, listener);
    }
}
