/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import java.io.Serializable;
import java.net.URI;
import java.net.URISyntaxException;

/**
 *
 * <p>
 * For testing purposes following constructor can be used:
 * 
 * new AgroURI("OTHER/1/TSK/2")
 * 
 * where TSK can be replaced by any value from the enum {@link ItemIdType}
 * </p>
 * 
 * 
 * @author Johan Reitzema
 */
public class AgroURI implements Serializable, EnterpriseIdProvider {
    public static final String DEFAULT_ISSUING_AGENCY = "AGRONL";
    
    private URI uri;
    private transient String[] parts;
    
    public AgroURI(String identifier) {
        assert identifier != null;
        try {
            this.uri = new URI(identifier);
        } catch (URISyntaxException ex) {
            throw new IllegalArgumentException(ex);
        }
        // TODO: validate format, move to factory.
    }
    
    private String[] getParts() {
        if (parts == null) {
            parts = uri.getPath().split("/");
            assert parts.length == 4;            
        }
        return parts;
    }
    
    /**
     * extract the issuing agency from the URI.
     * for the Netherlands this will always be AGRONL.
     * 
     * @return 
     */
    @Override
    public IssuingAgency getIssuingAgency() {
        return IssuingAgency.valueOf(getParts()[0]);
    }
    
    /**
     * extract the enterprise id from the URI.
     * for AGRONL this will be the KvK-number.
     * 
     * @return 
     */
    @Override
    public String getEnterpriseId() {
        return getParts()[1];
    }
    
    /**
     * extract the entity type from the URI.
     * 
     * @return 
     */
    public ItemIdType getItemIdType() {
        return ItemIdType.valueOf(getParts()[2]);
    }
    
    /**
     * extract the entity id from the URI.
     * 
     * @return 
     */
    public String getItemIdNumber() {
        return getParts()[3];
    }
    
    @Override
    public String toString() {
        return uri.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (obj instanceof AgroURI) {
            return uri.equals(((AgroURI)obj).uri);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return uri.hashCode();
    }
    
}
