/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import org.netbeans.api.project.Project;

/**
 *
 * @author johan
 */
public class AgroURIFactory {
    
    private final String baseURI;
    
    public AgroURIFactory(IssuingAgency issuingAgency, String enterpriseId) {
        assert issuingAgency != null;
        assert enterpriseId != null;                
        this.baseURI = issuingAgency.name() + "/" + enterpriseId + "/";
    }
    
    // create factory using e.g a Farm or an existing URI
    public AgroURIFactory(EnterpriseIdProvider enterpriseIdProvider) {
        this(enterpriseIdProvider.getIssuingAgency(), enterpriseIdProvider.getEnterpriseId());
    }
            
    public AgroURI createURI(ItemIdType itemIdType, String itemIdNumber) {
        assert itemIdType != null;
        assert itemIdNumber != null;
        
        if (!isValidItemIdNumber(itemIdNumber)) {
            throw new IllegalArgumentException("invalid ItemIdNumber");// TODO: to bundle
        }
        
        StringBuilder sb = new StringBuilder(baseURI)
                .append(itemIdType.name()).append("/").append(itemIdNumber);
        return new AgroURI(sb.toString());
    }

    public AgroURI createURI(ItemIdType itemIdType) {
        assert itemIdType != null;
        long id = EntityIdGenerator.Locator.getInstance().generateId(itemIdType);
        return createURI(itemIdType, Long.toString(id));
    }
    
    void validate() {
        
    }
        
    /**
     * validate itemIdNumber format.
     * EDI-teelt++ spec: an..14
     * @param itemIdNumber
     * @return 
     */
    boolean isValidItemIdNumber(String itemIdNumber) {
        if (itemIdNumber == null || itemIdNumber.equals("")) {
            return false;
        }
        
        //TODO: use pattern; spec needs clarification: example includes non alphanumeric character '.'
        int len = itemIdNumber.length();
        return len > 0 && len <= 14;
    }
    
    public static AgroURIFactory create(Project project) {
        EnterpriseIdProvider eip = project.getLookup().lookup(EnterpriseIdProvider.class);
        if (eip == null) {
            throw new RuntimeException("FIXME: Project should have an EnterpriseIdProvider in its lookup");
        }
        return new AgroURIFactory(eip);
    }
    
}
