/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import org.netbeans.api.project.Project;
import org.netbeans.api.project.ui.OpenProjects;
import org.openide.util.Lookup;

/**
 * Shared namespace for all entities within a single company/project/farm.
 * based on first two fields of URI path.
 *
 * @author johan
 */
public interface EnterpriseIdProvider {
    IssuingAgency getIssuingAgency();
    String getEnterpriseId();
    
    public static abstract class ProjectFinder {
        public static ProjectFinder getDefault() {
            ProjectFinder finder = Lookup.getDefault().lookup(ProjectFinder.class);
            if (finder == null) {
                finder = new OpenProjectFinder();
            }
            return finder;
        }
        
        public static Project find(EnterpriseIdProvider eip) {
            return getDefault().findProject(eip);
        }
        
        public abstract Project findProject(EnterpriseIdProvider eip);
    }
    
    public static class OpenProjectFinder extends ProjectFinder{

        @Override
        public Project findProject(EnterpriseIdProvider eip) {
            for (Project project : OpenProjects.getDefault().getOpenProjects()) {
                EnterpriseIdProvider projectEip = project.getLookup().lookup(EnterpriseIdProvider.class);
                if (projectEip != null &&
                        projectEip.getIssuingAgency() == eip.getIssuingAgency() &&
                        projectEip.getEnterpriseId().equals(eip.getEnterpriseId())) {
                    return project;
                }
            }
            return null;
        }
    }
    
    public static class Util {
        
        public static boolean isSameFarm(EnterpriseIdProvider eip1, EnterpriseIdProvider eip2) {
            if (eip1 == null || eip2 == null) {
                return false;
            }
            return eip1.getIssuingAgency() == eip2.getIssuingAgency()
                    && eip1.getEnterpriseId().equals(eip2.getEnterpriseId());
        }
        
        public static boolean isSameFarm(Entity e1, Entity e2) {
            if (e1 == null || e2 == null) {
                return false;
            }
            return isSameFarm(e1.getURI(), e2.getURI());
        }
    }
}
