/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import org.openide.util.NbBundle;

/**
 * Issuing Agency enum Can validate a issuing agency number
 *
 * @author johan
 */
@NbBundle.Messages({"IssuingAgency AGRONL=Kvk number should contain 8 digits"})
public enum IssuingAgency {

    AGRONL("\\d{8}"),
    OTHER(null);
    private static final String BUNDLE_PREFIX = "IssuingAgency ";

    private IssuingAgency(String EnterpriseIdPattern) {
        this.enterpriseIdPattern = EnterpriseIdPattern;
    }
    private final String enterpriseIdPattern;

    public String getEnterpriseIdPattern() {
        return enterpriseIdPattern;
    }

    /**
     * <p>Get message which describes the pattern</p> For instance, 'some
     * pattern should be n digits '
     *
     * @return
     */
    public String getPatternMessage() {
        return this.equals(IssuingAgency.OTHER) ? "" : NbBundle.getMessage(getClass(), BUNDLE_PREFIX + toString());
    }

    /**
     * validate enterpriseId format. EDI-teelt++ spec: depends on issuing agency
     * (e.g. KvK-number for AGRONL: n8)
     *
     * @param enterpriseId
     * @return
     */
    public boolean isValidEnterpriseId(String enterpriseId) {
        // should not be empty:
        if (enterpriseId == null || enterpriseId.isEmpty()) {
            return false;
        }

        // should match pattern (if any):
        if (enterpriseIdPattern != null && !enterpriseId.matches(enterpriseIdPattern)) {
            return false;
        }

        return true;
    }
}
