/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import org.openide.util.NbPreferences;
import org.openide.util.lookup.ServiceProvider;

/**
 * Simple implementation of an EntityIdGenerator.
 * 
 * Stores last generated id in module preferences.
 * Clearing the user directory or fresh installation will reset the
 * id.
 * 
 * @author Timon Veenstra <monezz@gmail.com>
 */
@ServiceProvider(service=EntityIdGenerator.class)
public class SimpleEntityIdGenerator implements EntityIdGenerator {
    
    private static final String PREF_LAST_GENERATED_ID = "lastGeneratedId";

    @Override
    public long generateId(ItemIdType itemIdType) {
        return generateId(itemIdType.name());
    }

    @Override
    public long generateId(String key) {
        long lastId = getLastId(key);
        lastId++;
        NbPreferences.forModule(this.getClass()).putLong(PREF_LAST_GENERATED_ID + key, lastId);
        return lastId;
    }
    
    private long getLastId(String key) {
        return NbPreferences.forModule(this.getClass()).getLong(PREF_LAST_GENERATED_ID + key, 0l);
    }
    
}
